/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// The code example below shows how to use regular expression functions:
//     ippsRegExpGetSize
//     ippsRegExpInit
//     ippsRegExpFind_8u


#include <stdio.h>
#include "ipp.h"

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) Integrated Primitives (Intel(R) IPP) function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

static Ipp8u string[] = "Hello World!";
int main(void)
{
   const char pattern[] = "[ hH ]ello (\\w+)";
   IppRegExpFind find[2] = {0};
   int find_num = sizeof (find)/ sizeof (find[0]);
   IppStatus status = ippStsNoErr;
   int i = 0, errOffset = 0;
   IppRegExpState *pRegExpState = NULL;
   int regExpStateSize = 0;

   check_sts( status = ippsRegExpGetSize( pattern, &regExpStateSize ) )

   pRegExpState = (IppRegExpState*)ippMalloc( regExpStateSize );

   check_sts( status = ippsRegExpInit( pattern, NULL, pRegExpState, &errOffset ) )

   check_sts( status = ippsRegExpFind_8u( string, sizeof (string) - 1, pRegExpState, find, &find_num ) )

   printf ( "Found strings number = %d\n", find_num );

   for( i = 0; i < find_num ; i ++ ) {
      Ipp8u tmp = ((Ipp8u*)find[i].pFind)[find[i].lenFind];
      ((Ipp8u*)find[i].pFind)[find[i].lenFind] = 0;
      printf ( "%d: %s\n", i, (char*)find[i].pFind );
      ((Ipp8u*)find[i].pFind )[find[i].lenFind ] = tmp;
   }

EXIT_MAIN
   printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
   return (int)status;
}
