/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

//   A simple example of performing the forward FFT for processing complex data.
// implemented with Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiFFTGetSize_C_32fc
//     ippiFFTInit_C_32fc
//     ippiFFTFwd_CToC_32fc_C1R

#include <stdio.h>
#include "ipp.h"

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{
    IppStatus           status;
    IppiFFTSpec_C_32fc  *pSpec = NULL;                              /* Pointer to FFT spec structure */

    Ipp32fc             srcFwd[8 * 8] = {0}, dstFwd[8 * 8] = {0};   /* Source/destination images */
    Ipp32fc             m3 = {-3, 0}, one = {1, 0};

    Ipp8u               *pMemInit = NULL, *pBuffer = NULL;          /* Pointer to the work buffers */
    int                 sizeSpec = 0, sizeInit = 0, sizeBuf = 0;    /* Size of FFT spec structure, init and work buffers */

    srcFwd[0] = m3;
    srcFwd[9] = one;

    check_sts( status = ippiFFTGetSize_C_32fc(3, 3, IPP_FFT_DIV_INV_BY_N, ippAlgHintAccurate, &sizeSpec, &sizeInit, &sizeBuf) )

    /* memory allocation */
    pSpec    = (IppiFFTSpec_C_32fc*) ippMalloc(sizeSpec);
    pBuffer  = (Ipp8u*) ippMalloc(sizeBuf);
    pMemInit = (Ipp8u*) ippMalloc(sizeInit);

    check_sts( status = ippiFFTInit_C_32fc(3, 3, IPP_FFT_DIV_INV_BY_N, ippAlgHintAccurate, pSpec, pMemInit) )

    /* forward FFT transform */
    check_sts( status = ippiFFTFwd_CToC_32fc_C1R(srcFwd, 8*sizeof(Ipp32fc), dstFwd, 8*sizeof(Ipp32fc), pSpec, pBuffer) )

EXIT_MAIN
    ippFree(pMemInit);
    ippFree(pSpec);
    ippFree(pBuffer);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}

