/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

//   A simple example of performing the forward FFT for processing real data.
// implemented with Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiFFTGetSize_R_32f
//     ippiFFTInit_R_32f
//     ippiFFTFwd_RToPack_32f_C1R


#include <stdio.h>
#include "ipp.h"

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{
    IppStatus status;
    IppiFFTSpec_R_32f *pSpec = NULL;  /* Pointer to FFT spec structure */

    /* Pointers to source/destination images */
    Ipp32f pSrcFwd[64] = {0}, pDstFwd[64] = {0};

    /* Pointer to the work buffers */
    Ipp8u *pMemInit = NULL;
    Ipp8u *pBuffer  = NULL;

    int sizeSpec = 0, sizeInit = 0, sizeBuf = 0; /* size of FFT spec structure, Init and work buffers */

    pSrcFwd[0] = -3; pSrcFwd[9] = 1;

    check_sts( status = ippiFFTGetSize_R_32f(3, 3, IPP_FFT_DIV_INV_BY_N, ippAlgHintAccurate,
                         &sizeSpec, &sizeInit, &sizeBuf) )

    /* memory allocation */
    pSpec    = (IppiFFTSpec_R_32f*) ippMalloc( sizeSpec );
    pBuffer  = (Ipp8u*) ippMalloc( sizeBuf );
    pMemInit = (Ipp8u*) ippMalloc( sizeInit );

    /* inverse FFT transform */
    check_sts( status = ippiFFTInit_R_32f(3, 3, IPP_FFT_DIV_INV_BY_N, ippAlgHintAccurate, pSpec, pMemInit) )

    check_sts( status = ippiFFTFwd_RToPack_32f_C1R( pSrcFwd, 8*sizeof(Ipp32f),
                                         pDstFwd, 8*sizeof(Ipp32f), pSpec, pBuffer ) )

EXIT_MAIN
    ippFree( pMemInit );
    ippFree( pSpec );
    ippFree( pBuffer );
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
