/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of performing Bilateral filtering of an image implemented with Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiFilterBilateralBorderGetBufferSize
//     ippiFilterBilateralBorderInit
//     ippiFilterBilateralBorder_8u_C1R


#include <stdio.h>
#include "ipp.h"

#define WIDTH  128  /* image width  */
#define HEIGHT  64  /* image height */

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{
    IppStatus status = ippStsNoErr;
    Ipp8u* pSrc = NULL, *pDst = NULL;     /* Pointers to source/destination images */
    int srcStep = 0, dstStep = 0;         /* Steps, in bytes, through the source/destination images */
    IppiSize roiSize = { WIDTH, HEIGHT }; /* Size of source/destination ROI in pixels */
    Ipp8u *pBuffer = NULL;                /* Pointer to the work buffer */
    IppiFilterBilateralSpec *pSpec = NULL;
    int iTmpBufSize = 0, iSpecSize = 0;   /* Common work buffer size */
    IppiBorderType borderType = (IppiBorderType)(ippBorderRepl | ippBorderInMemTop | ippBorderInMemRight);
    Ipp8u borderValue = 0;
    int radius = 1; /* Radius of circular neighborhood what defines pixels for calculation. */
    int numChannels = 1;
    Ipp32f valSqrSigma = 400.0f, posSqrSigma = 100.0f; /* pixel intensity/position */

    pSrc = ippiMalloc_8u_C1(roiSize.width, roiSize.height, &srcStep);
    pDst = ippiMalloc_8u_C1(roiSize.width, roiSize.height, &dstStep);

    check_sts( status = ippiFilterBilateralBorderGetBufferSize(ippiFilterBilateralGauss, roiSize, radius, ipp8u, numChannels, ippDistNormL1, &iSpecSize, &iTmpBufSize) )

    pSpec = (IppiFilterBilateralSpec *)ippsMalloc_8u(iSpecSize);

    pBuffer = ippsMalloc_8u(iTmpBufSize);

    check_sts( status = ippiFilterBilateralBorderInit(ippiFilterBilateralGauss, roiSize, radius, ipp8u, 1, ippDistNormL1, valSqrSigma, posSqrSigma, pSpec) )

    check_sts( status = ippiFilterBilateralBorder_8u_C1R(pSrc + srcStep, srcStep, pDst, dstStep, roiSize, borderType, &borderValue, pSpec, pBuffer) )

EXIT_MAIN
    ippsFree(pBuffer);
    ippsFree(pSpec);
    ippiFree(pSrc);
    ippiFree(pDst);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
