/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

//   A simple example of performing filtering an image using a general integer rectangular kernel
// implemented with Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ipprFilterBorderGetSize
//     ipprFilterBorderInit_16s
//     ipprFilterBorder_8u_C1R


#include <stdio.h>
#include "ipp.h"

#define WIDTH  128  /* image width */
#define HEIGHT  64  /* image height */
#define DEPTH  64  /* image depth */
const Ipp16s kernel[3*3*3] = { 0, 1, 0, 1, 0, 1, 0, 1, 0,
                               0, 1, 0, 1, 0, 1, 0, 1, 0 ,
                               0, 1, 0, 1, 0, 1, 0, 1, 0 };

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{
    IppStatus status = ippStsNoErr;
    Ipp8u* pSrc = NULL, *pDst = NULL;     /* Pointers to source/destination images */
    IppSizeL srcPlaneStep =0, srcStep = 0, dstPlaneStep, dstStep = 0;         /* Steps, in bytes, through the source/destination images */
    IpprVolumeL  roiVolume = { WIDTH, HEIGHT, DEPTH }; /* Size of source/destination ROI in pixels */
    IpprVolumeL  kernelVolume = { 3, 3, 3};
    Ipp8u *pBuffer = NULL;                /* Pointer to the work buffer */
    IpprFilterBorderSpec* pSpec = NULL;   /* context structure */
    IppSizeL iTmpBufSize = 0, iSpecSize = 0;   /* Common work buffer size */
    IpprBorderType borderType = ipprBorderRepl;
    Ipp8u borderValue = 0;
    int numChannels = 1;

    pSrc = ippsMalloc_8u_L(roiVolume.width*roiVolume.height*roiVolume.depth);
    pDst = ippsMalloc_8u_L(roiVolume.width*roiVolume.height*roiVolume.depth);
    srcStep      = roiVolume.width;
    srcPlaneStep = roiVolume.height * srcStep;
    dstStep      = roiVolume.width;
    dstPlaneStep = roiVolume.height * srcStep;

    check_sts( status = ipprFilterBorderGetSize_L(kernelVolume, roiVolume, ipp8u, ipp16s, numChannels, &iSpecSize, &iTmpBufSize) )

    pSpec = (IpprFilterBorderSpec *)ippsMalloc_8u(iSpecSize);
    pBuffer = ippsMalloc_8u_L(iTmpBufSize);

    check_sts( status = ipprFilterBorderInit_16s_L(kernel, kernelVolume, 4, ipp8u, numChannels, pSpec) )

    check_sts(status = ipprFilterBorder_8u_C1V_L(pSrc, srcPlaneStep, srcStep, pDst, dstPlaneStep, dstStep, roiVolume, borderType, &borderValue, pSpec, pBuffer))

EXIT_MAIN
    ippsFree(pBuffer);
    ippsFree(pSpec);
    ippiFree(pSrc);
    ippiFree(pDst);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
