/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

//Perform linear filtering of an image using one of
//  predefined convolution kernels.
//     ippiFilterSobelVertSecondBorderGetBufferSize
//     ippiFilterSobelVertSecondBorder_8u16s_C1R

#include <stdio.h>
#include "ipp.h"

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) Integrated Primitives (Intel(R) IPP) function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{

    IppStatus status = ippStsNoErr;
    Ipp8u pSrc[8*8]={
                     4, 4, 4, 4, 4, 4, 4, 4,
                     4, 3, 3, 3, 3, 3, 3, 4,
                     4, 3, 2, 2, 2, 2, 3, 4,
                     4, 3, 2, 1, 1, 2, 3, 4,
                     4, 3, 2, 1, 1, 2, 3, 4,
                     4, 3, 2, 2, 2, 2, 3, 4,
                     4, 3, 3, 3, 3, 3, 3, 4,
                     4, 4, 4, 4, 4, 4, 4, 4
                   };
    Ipp16s pDst[8*8];
    IppiSize roiSize = { 8, 8 };
    Ipp8u borderValue = 3;
    int pBufferSize = 0;
    Ipp8u* pBuffer = NULL;

    check_sts( status = ippiFilterSobelVertSecondBorderGetBufferSize(roiSize, ippMskSize3x3,
       ipp8u, ipp16s, 1, &pBufferSize))

    pBuffer = ippiMalloc_8u_C1 ( 8, 8, &pBufferSize );

    check_sts( status = ippiFilterSobelVertSecondBorder_8u16s_C1R ( pSrc, 8, pDst, 8*sizeof(Ipp16s), roiSize,
                         ippMskSize3x3, ippBorderConst, borderValue, pBuffer ) )

EXIT_MAIN
    ippsFree(pBuffer);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return status;
}
/*
    Result:

    4 4 4 4 4 4 4 4
    4 3 3 3 3 3 3 4
    4 3 2 2 2 2 3 4
    4 3 2 1 1 2 3 4
    4 3 2 1 1 2 3 4     pSrc
    4 3 2 2 2 2 3 4
    4 3 3 3 3 3 3 4
    4 4 4 4 4 4 4 4


    -4 1 0 0 0 0 1 -4
    -7 2 1 0 0 1 2 -7
    -8 1 2 1 1 2 1 -8
    -8 0 1 3 3 1 0 -8
    -8 0 1 3 3 1 0 -8   pDst
    -8 1 2 1 1 2 1 -8
    -7 2 1 0 0 1 2 -7
    -4 1 0 0 0 0 1 -4

*/
