/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

//   A simple example of performing two-dimensional adaptive noise-removal filtering of an image using Wiener filter
// implemented with Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiFilterWienerGetBufferSize
//     ippiFilterWiener_32f_C1R


#include <stdio.h>
#include "ipp.h"

#define WIDTH  128  /* image width */
#define HEIGHT  64  /* image height */

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{
    IppStatus status = ippStsNoErr;
    Ipp32f* pSrc = NULL, *pDst = NULL;    /* Pointers to source/destination images */
    Ipp32f* pSrcW = NULL;                 /* Pointers to source ROI image */
    int srcStep, dstStep;                 /* Steps, in bytes, through the source/destination images */
    IppiSize roiSize = { WIDTH, HEIGHT }; /* Size of source/destination in pixels */
    IppiSize maskSize = { 3, 3 };
    IppiSize dstRoiSize = {0};            /* Size of destination ROI in pixels */
    IppiPoint anchor = { 1, 1 };          /* Anchor cell specifying the mask alignment\
                                          with respect to the position of the input pixel */
    Ipp32f noise[1] = { 0.0 };            /* Noise level value */
    Ipp8u *pBuffer = NULL;                /* Pointer to the work buffer */
    int iTmpBufSize = 0;                  /* Common work buffer size */
    int numChannels = 1;

    dstRoiSize.width  = WIDTH  - maskSize.width;
    dstRoiSize.height = HEIGHT - maskSize.height;

    /* memory allocation */
    pSrc = ippiMalloc_32f_C1(roiSize.width, roiSize.height, &srcStep);
    pDst = ippiMalloc_32f_C1(roiSize.width, roiSize.height, &dstStep);

    check_sts( status = ippiImageJaehne_32f_C1R(pSrc, srcStep, roiSize) ) /* fill source image */

    check_sts( status = ippiFilterWienerGetBufferSize(dstRoiSize, maskSize, numChannels, &iTmpBufSize) )

    check_sts( status = ippiSet_32f_C1R(1.f, pDst, dstStep, roiSize) )

    pBuffer = ippsMalloc_8u(iTmpBufSize);
    pSrcW = (Ipp32f*)((Ipp8u*)(pSrc + anchor.x) + anchor.y * srcStep);

    check_sts( status = ippiFilterWiener_32f_C1R(pSrcW, srcStep, pDst, dstStep, dstRoiSize, maskSize, anchor, noise, pBuffer) )

EXIT_MAIN
    ippsFree(pBuffer);
    ippiFree(pSrc);
    ippiFree(pDst);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
