/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of finding coordinates of peaks
//     ippiFindPeaks3x3GetBufferSize_32f_C1R
//     ippiFindPeaks3x3_32f_C1R


#include <stdio.h>
#include "ipp.h"

#define WIDTH  128  /* Image width  */
#define HEIGHT 128  /* Image height */
#define KERSIZE 3   /* Kernel size  */

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) Integrated Primitives (Intel(R) IPP) function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{
    IppStatus status = ippStsNoErr;
    Ipp32f *pSrc = NULL;       /*  initial image  */
    Ipp8u *pBuffer = NULL;     /*  working buffer */
    IppiPoint *pPeak = NULL;   /*  array of peaks */
    IppiSize roiSize = { WIDTH, HEIGHT };    /* image size */
    int srcStep = 0;           /*  row step in bytes */
    int bufSize = 0;           /*  working buffer size */
    int maxPeaks=500;          /*  max peaks number for image */
    int peakCount = 0;         /*  peaks number for image */
    Ipp32f threshold=0.02f;    /*  peak threshold */
    Ipp32s borderValue=40;     /*  Border Value */


    pSrc = ippiMalloc_32f_C1(roiSize.width, roiSize.height, &srcStep);

    check_sts( status = ippiFindPeaks3x3GetBufferSize_32f_C1R(roiSize.width,&bufSize) )

    pBuffer = ippsMalloc_8u(bufSize);
    pPeak   = (IppiPoint*)ippsMalloc_8u(maxPeaks*sizeof(IppiPoint));

    check_sts( status = ippiFindPeaks3x3_32f_C1R(pSrc,srcStep,roiSize,threshold,pPeak,maxPeaks,
          &peakCount,ippiNormL1,borderValue,pBuffer) )

EXIT_MAIN
    ippsFree(pBuffer);
    ippsFree(pPeak);
    ippiFree(pSrc);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return status;
}
