/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of gradient flood filling of connected area
// using Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiFloodFillGetSize_Grad
//     ippiFloodFill_Grad4Con_8u_C1IR


#include <stdio.h>
#include "ipp.h"

#define WIDTH   8  /* source image width */
#define HEIGHT  8  /* source image height */

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

static Ipp8u pImage[WIDTH*HEIGHT] = /* Pointer to ROI of initial image */
{
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7
};

int main(void)
{
    IppStatus status = ippStsNoErr;
    int imageStep = WIDTH;              /* Steps, in bytes, through the source images */
    IppiSize roiSize = { WIDTH, HEIGHT};/* Size of source ROI in pixels */
    IppiConnectedComp pRegion;          /* Pointer to repainted region properties structure */
    IppiPoint seedPoint = {1,0};        /* Coordinates of the seed point inside image ROI */
    Ipp8u PointNewValue = 1;            /* Value to fill with for one-channel data */
    Ipp8u* pBuffer = NULL;              /* Pointer to the work buffer */
    Ipp8u minDelta=1, maxDelta=1;       /* Minimum/maximum difference between neighbour pixels for one-channel data */
    int bufferSize = 0;

    /* Calculate size of temporary buffer for flood filling operation */
    check_sts( status = ippiFloodFillGetSize_Grad(roiSize, &bufferSize) )

    pBuffer = ippsMalloc_8u(bufferSize);

    /* Perform gradient flood filling of 4-connected area */
    check_sts( status = ippiFloodFill_Grad4Con_8u_C1IR(pImage, imageStep, roiSize, seedPoint, PointNewValue, minDelta, maxDelta, &pRegion, pBuffer) )

EXIT_MAIN
    ippsFree(pBuffer);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
