/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of labelling connected non-zero components with 
// different label values using Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiLabelMarkersGetBufferSize_16u_C1R
//     ippiLabelMarkers_16u_C1IR


#include <stdio.h>
#include "ipp.h"

#define WIDTH   8  /* source image width */
#define HEIGHT  8  /* source image height */

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

static Ipp16u pMarker[WIDTH*HEIGHT] = /* Pointer to source and destination marker image */
{ 0, 12, 13,  0,  0,  0,  0,  0,
  0, 22, 23, 24,  0,  0,  0,  0,
  0, 32, 33, 34,  0,  0,  0,  0,
  0, 42, 43, 44,  0,  0,  0,  0,
  0,  0, 53, 54,  0, 56, 57, 58,
  0,  0,  0,  0,  0, 66,  0, 68,
  0,  0,  0, 57,  0, 76, 77, 78,
  0,  0,  0,  0, 68,  0,  0,  0
};

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */



int main(void)
{
    IppStatus status = ippStsNoErr;
    int markerStep = WIDTH*sizeof(Ipp16u); /* Steps, in bytes, through the source images */
    IppiSize roiSize = { WIDTH, HEIGHT };  /* Size of source ROI in pixels */
    int minLabel = 1;                      /* Minimal label value */
    int maxLabel = 30000;                  /* Maximal label value */
    int markersNum = 0;                    /* Pointer to number of markers */
    Ipp8u* pBuffer = NULL;                 /* Pointer to the work buffer */
    int bufferSize = 0;

    /* Calculate size of temporary buffer */
    check_sts( status = ippiLabelMarkersGetBufferSize_16u_C1R(roiSize, &bufferSize) )

    pBuffer = ippsMalloc_8u(bufferSize);

    /* Label connected non-zero components with different label values */
    check_sts( status = ippiLabelMarkers_16u_C1IR(pMarker, markerStep, roiSize, minLabel, maxLabel, ippiNormL1, &markersNum, pBuffer) )

EXIT_MAIN
    ippsFree(pBuffer);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
