/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of Line Suppression 
// using Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiLineSuppressionGetBufferSize
//     ippiLineSuppression_8u_C1R


#include <stdio.h>
#include "ipp.h"

#define WIDTH  128  /* image width  */
#define HEIGHT  64  /* image height */

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{
    IppStatus status = ippStsNoErr;
    Ipp8u* pSrc = NULL, *pDst = NULL, *pFeature = NULL;  /* Pointers to source/destination/feature images */
    int srcStep = 0, dstStep = 0, featureStep = 0;       /* Steps, in bytes, through the source/destination/feature images */
    IppiSize roiSize = { WIDTH, HEIGHT };    /* Size of source/destination ROI in pixels */
    IppiMaskSize filterMask = ippMskSize3x3; /* Linear size of derivative filter aperture */
    Ipp32u avgWndSize = 1;                   /* Neighbourhood block size for smoothing */
    Ipp32u numChannels = 1;
    Ipp32f threshold = 5.0f;
    const Ipp8u borderValue = 0;
    Ipp8u* pBuffer = NULL;                   /* Pointer to the work buffer */
    int bufSize = 0;
    IppiDifferentialKernel filterType = ippFilterSobel; /* Type of derivative operator */

    pSrc = ippiMalloc_8u_C1(roiSize.width, roiSize.height, &srcStep);
    pDst = ippiMalloc_8u_C1(roiSize.width, roiSize.height, &dstStep);
    pFeature = ippiMalloc_8u_C1(roiSize.width, roiSize.height, &featureStep);

    /* Computes the temporary work buffer size */
    check_sts( status = ippiLineSuppressionGetBufferSize(roiSize, filterMask, avgWndSize, ipp8u, numChannels, &bufSize) )
    pBuffer = ippsMalloc_8u(bufSize);

    /* Line Suppression processing */
    check_sts( status = ippiLineSuppression_8u_C1R(pSrc, srcStep, pFeature, featureStep, pDst, dstStep, roiSize, filterType, filterMask, avgWndSize, threshold, ippBorderRepl, borderValue, pBuffer) )

EXIT_MAIN
    ippiFree(pSrc);
    ippiFree(pDst);
    ippiFree(pFeature);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
