/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of performing filtering of small noise blobs (speckles)
// in the image using Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiMarkSpecklesGetBufferSize
//     ippiMarkSpeckles_8u_C1IR


#include <stdio.h>
#include "ipp.h"

#define WIDTH  128  /* image width  */
#define HEIGHT  64  /* image height */

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{
    IppStatus status = ippStsNoErr;
    int numChannels = 1;
    Ipp8u* pSrcDst = NULL;                /* Pointers to source/destination images */
    int srcDstStep = 0;                   /* Steps, in bytes, through the source/destination images */
    IppiSize roiSize = { WIDTH, HEIGHT }; /* Size of source/destination ROI in pixels */
    Ipp8u  speckleVal = 255;              /* Disparity value */
    Ipp8u  maxPixDif = 1;                 /* Maximum difference between neighbor pixels */
    Ipp32s maxSpeckleSize = 8;            /* Maximum difference between neighbor pixels */
    Ipp8u* pBuffer = NULL;                /* Pointer to the work buffer */
    int bufferSize = 0;

    /* Computes the temporary work buffer size */
    check_sts( status = ippiMarkSpecklesGetBufferSize(roiSize, ipp8u, numChannels, &bufferSize) )

    pBuffer = ippsMalloc_8u(bufferSize);
    pSrcDst = ippiMalloc_8u_C1(roiSize.width, roiSize.height, &srcDstStep);

    check_sts( status = ippiMarkSpeckles_8u_C1IR(pSrcDst, srcDstStep, roiSize, speckleVal, maxSpeckleSize, maxPixDif, ippiNormL1, pBuffer) )

EXIT_MAIN
    ippiFree(pSrcDst);
    ippsFree(pBuffer);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
