/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of mirroring an image about
// the ippAxs45 and ippAxs135 axis using Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiMirror_8u_C1R


#include <stdio.h>
#include "ipp.h"

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

int main(void)
{
    IppStatus status = ippStsNoErr;
    Ipp8u pSrc[8 * 4] = {/* Pointer to source images */
        1, 2, 3, 4, 8, 8, 8, 8,
        1, 2, 3, 4, 8, 8, 8, 8,
        1, 2, 3, 4, 8, 8, 8, 8,
        1, 2, 3, 4, 8, 8, 8, 8 };
    Ipp8u pDst[8 * 8] = {/* Pointer to destination images */
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0};
    int srcStep = 8, dstStep = 8; /* Steps, in bytes, through the source/destination images */
    IppiSize roiSize = { 3, 4 };  /* Size of source/destination ROI in pixels */
    IppiAxis flip1 = ippAxs45;    /* The axis to mirror the image about */
    IppiAxis flip2 = ippAxs135;   /* The axis to mirror the image about */

    check_sts( status = ippiMirror_8u_C1R(pSrc, srcStep, pDst, dstStep, roiSize, flip1) )
    check_sts( status = ippiMirror_8u_C1R(pSrc, srcStep, pDst, dstStep, roiSize, flip2) )

EXIT_MAIN
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
