/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of gradient flood filling of connected area
// using Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiMulC64f_8u_C1R
//     


#include <stdio.h>
#include "ipp.h"

#define WIDTH   8  /* source image width */
#define HEIGHT  8  /* source image height */

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

static Ipp8u pImage[WIDTH*HEIGHT] = /* Pointer to ROI of initial image */
{
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7,
    0, 1, 2, 3, 4, 5, 6, 7
};

int main(void)
{
    IppStatus status = ippStsNoErr;
    int imageStep = WIDTH;              /* Steps, in bytes, through the source and destination images */
    Ipp8u pDstImage[WIDTH*HEIGHT];
    IppiSize roiSize = { WIDTH, HEIGHT};/* Size of source ROI in pixels */
    Ipp64f value[1] = {0.5};
    /* Calculate size of temporary buffer for flood filling operation */

    /* Perform gradient flood filling of 4-connected area */
    check_sts( status = ippiMulC64f_8u_C1R(pImage, imageStep, value, pDstImage, imageStep, roiSize, ippAlgHintFast, ippRndNear) )

EXIT_MAIN
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
