/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of the source image transformation by remapping
// of its pixels using Intel(R) Integrated Primitives (Intel(R) IPP) function:
//     ippiRemap_8u_C1R


#include <stdio.h>
#include "ipp.h"

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

int main(void)
{
    IppStatus status = ippStsNoErr;
    Ipp8u pSrc[4 * 4] =/* Pointers to source images */
    {   0, 1, 2, 3,
        4, 5, 6, 7,
        8, 9, 0, 1,
        2, 3, 4, 5 };
    Ipp8u pDst[4*4];              /* Pointers to destination images */
    int srcStep = 4, dstStep = 4; /* Steps, in bytes, through the source/destination images */
    IppiSize roiSize = { 4, 4 };  /* Size of source/destination ROI in pixels */
    Ipp32f pxMap[4 * 4] = {/* Pointer to x map */
        0, 1, 2, 3,
        0, 1, 2, 0,
        0, 1, 0, 0,
        0, 1, 2, 3 };
    Ipp32f pyMap[4 * 4] = {/* Pointer to y map */
        0, 0, 0, 0,
        1, 1, 1, 0,
        2, 2, 0, 0,
        3, 3, 3, 3 };
    int xStep = 16, yStep = 16;     /* Steps, in bytes, through the x and y maps */
    IppiRect rect = { 0, 0, 4, 4 }; /* Region of interest in the source image */

    check_sts( status = ippiRemap_8u_C1R(pSrc, roiSize, srcStep, rect, pxMap, xStep, pyMap, yStep,
        pDst, dstStep, roiSize, IPPI_INTER_LINEAR) )

EXIT_MAIN
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
