/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of changing an image size by different interpolation 
// methods with antialiasing technique using Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiResizeGetSize_8u
//     ippiResizeAntialiasingLinearInit
//     ippiResizeGetBufferSize_8u
//     ippiResizeAntialiasing_8u_C3R


#include <stdio.h>
#include "ipp.h"

#define SRC_WIDTH  128  /* source image width */
#define SRC_HEIGHT  64  /* source image height */
#define DST_WIDTH   64  /* source image width */
#define DST_HEIGHT 128  /* source image height */

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{
    IppStatus status = ippStsNoErr;
    Ipp8u* pSrc = NULL, *pDst = NULL;   /* Pointers to source/destination images */
    int srcStep = 0, dstStep = 0;       /* Steps, in bytes through the source/destination images */
    IppiSize srcSize = { SRC_WIDTH, SRC_HEIGHT }; /* Size of source ROI in pixels */
    IppiSize dstSize = { DST_WIDTH, DST_HEIGHT }; /* Size of destination ROI in pixels */
    Ipp8u *pBuffer = NULL;              /* Pointer to the work buffer */
    IppiResizeSpec_32f *pSpec = NULL;
    Ipp8u *pInit = NULL;
    int specSize = 0, initSize = 0, bufSize = 0;
    IppiPoint dstOffset = { 0, 0 };
    IppiInterpolationType interpolation = ippLinear;
    IppiBorderType borderType = ippBorderRepl;

    pSrc = ippiMalloc_8u_C3(srcSize.width, srcSize.height, &srcStep);
    pDst = ippiMalloc_8u_C3(dstSize.width, dstSize.height, &dstStep);

    /* Calculation of work buffer size */
    check_sts( status = ippiResizeGetSize_8u(srcSize, dstSize, interpolation, 1, &specSize, &initSize) )

    /* Memory allocation */
    pSpec = (IppiResizeSpec_32f*)ippsMalloc_8u(specSize + initSize);

    pInit = (Ipp8u*)pSpec + initSize;

    /* Filter initialization */
    check_sts( status = ippiResizeAntialiasingLinearInit(srcSize, dstSize, pSpec, pInit) )

    check_sts( status = ippiResizeGetBufferSize_8u(pSpec, dstSize, 3, &bufSize) )

    pBuffer = ippsMalloc_8u(bufSize);

    /* Function call */
    check_sts( status = ippiResizeAntialiasing_8u_C3R(pSrc, srcStep, pDst, dstStep, dstOffset, dstSize, borderType, 0, pSpec, pBuffer) )

EXIT_MAIN
    ippiFree(pSrc);
    ippiFree(pDst);
    ippsFree(pSpec);
    ippsFree(pBuffer);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
