/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

//    The code example below illustrates how to use the ippiScaleC function.

#include <stdio.h>
#include "ipp.h"


/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) Integrated Primitives (Intel(R) IPP) function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */


int main(void)
{
    IppStatus status = ippStsNoErr;

    Ipp32u seed=0;
    IppiSize roiSize = {8, 8}; /* Image sizes */
    Ipp8u pSrc[8*8]={0};       /* Pointers to source images */
    Ipp16s pDst[8*8];          /* Pointers to destination images */
    Ipp64f mVal, aVal;
    Ipp64f minSrc=IPP_MIN_8U, maxSrc=IPP_MAX_8U;
    Ipp64f minDst=IPP_MIN_16S, maxDst=IPP_MAX_16S;

    /* set random image  */
    check_sts( status = ippiAddRandUniform_8u_C1IR( pSrc, roiSize.width*sizeof(*pSrc), roiSize, IPP_MIN_8U, IPP_MAX_8U, &seed) )

    /* calculate coefficient for scaling */
    mVal = (maxDst-minDst) / (maxSrc-minSrc);
    aVal = minDst - minSrc*mVal;

    /* do scaling */
    check_sts( status = ippiScaleC_8u16s_C1R(pSrc, roiSize.width*sizeof(*pSrc), mVal, aVal, pDst, roiSize.width*sizeof(*pDst), roiSize, ippAlgHintFast) )

EXIT_MAIN
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}

