/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

#include <stdio.h>
#include "ipp.h"
#include <math.h>

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) Integrated Performance Primitives (Intel(R) IPP) function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main()
{
    int i;
    IppStatus status;
    Ipp64f cart[6], magn[4], phase[4];
    for (i = 0; i<6; ++i) cart[i] = sin(IPP_2PI * i / 8);

    printf("Source:\n");
    printf("\nre= ");
    for (i = 0; i < 4; i++) printf("%f ", cart[i]);
    printf("\nim= ");
    for (i = 2; i < 6; i++) printf("%f ", cart[i]);

    check_sts(status = ippsCartToPolar_64f(cart, cart + 2, magn, phase, 4));

    printf("\nResult:");
    printf("\nmagn= ");
    for (i = 0; i < 4; i++) printf("%f ", magn[i]);
    printf("\nphase= ");
    for (i = 0; i < 4; i++) printf("%f ", phase[i]);

EXIT_MAIN
    printf("\nExit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
