/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

#include <stdio.h>
#include "ipp.h"

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel Integrated Performance Primitives (Intel(R) IPP) function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main()
{
    int len = 32;
    int i;
    Ipp64f *pDst = ippsMalloc_64f(len * sizeof(Ipp64f));
    Ipp64f *pSrc1 = ippsMalloc_64f(len * sizeof(Ipp64f));
    Ipp64f *pSrc2 = ippsMalloc_64f(len * sizeof(Ipp64f));
    IppStatus status;

    printf("\n\nSource vector 1\n");
    for (i = 0; i < len; i++)
    {
        pSrc1[i] = i;
        printf("%.0f; ", pSrc1[i]);
    }
    printf("\n\nSource vector 2\n");
    for (i = 0; i < len; i++)
    {
        pSrc2[i] = 10 + i;
        printf("%.0f; ", pSrc2[i]);
    }

    check_sts(status = ippsDiv_64f(pSrc2, pSrc1, pDst, len));

    printf("\n\nResult\n");
    for (i = 0; i < len; i++) printf("%.2f; ", pDst[i]);
    printf("\n\n");

EXIT_MAIN
    ippsFree(pSrc1);
    ippsFree(pSrc2);
    ippsFree(pDst);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
