/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

#include <math.h>
#include <stdio.h>
#include "ipp.h"

#undef NUMITERS
#define NUMITERS 150

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) Integrated Primitives (Intel(R) IPP) function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main()
{
    int n = 0, i = 0, specSize = 0, bufferSize = 0;
    IppStatus status;
    IppsIIRState_32f *pIIRState = NULL;  /* Pointer to Intel(R) IPP state structure */
    IppsFIRSpec_32f  *pFIRSpec  = NULL;  /* Pointer to FIR Spec structure */
    Ipp32f *x = ippsMalloc_32f(NUMITERS + 10),
           *y = ippsMalloc_32f(NUMITERS),
           *z = ippsMalloc_32f(NUMITERS);
    const float taps[] = {
        0.0051f, 0.0180f, 0.0591f, 0.1245f, 0.1869f, 0.2127f, 0.1869f,
        0.1245f, 0.0591f, 0.0180f, 0.0051f, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    };
    Ipp8u* pBuf = NULL; /* Pointer to the work buffers */

    for (n = 0; n < NUMITERS; ++n) x[n] = (float)sin(IPP_2PI *n *0.2);

    check_sts( status = ippsIIRGetStateSize_32f(10, &bufferSize) )

    pBuf = ippsMalloc_8u(bufferSize);

    check_sts( status = ippsIIRInit_32f(&pIIRState, taps, 10, NULL, pBuf) )

    check_sts( status = status = ippsIIR_32f(x, y, NUMITERS, pIIRState) )

    ippsFree(pBuf); pBuf = NULL;

    for (i = 120; i < 120+5; i++) printf("%f,", y[i]);
    printf("\n");

    check_sts( status = ippsFIRSRGetSize(11, ipp32f, &specSize, &bufferSize) )
    pFIRSpec   = (IppsFIRSpec_32f *)ippsMalloc_8u(specSize);
    pBuf   =                        ippsMalloc_8u(bufferSize);

    check_sts( status = ippsFIRSRInit_32f(taps, 11, ippAlgDirect, pFIRSpec) )
    check_sts( status = ippsFIRSR_32f(x, z, NUMITERS, pFIRSpec, NULL, NULL, pBuf) )

    for (i = 120; i < 120+5; i++) printf("%f,", z[i]);

    printf("\n");
EXIT_MAIN
    ippsFree(pBuf);
    ippsFree(z);
    ippsFree(y);
    ippsFree(x);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}


