/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
// A simple example of performing Abs functions:
//     ippsqAbs_32f
//     ippsqAbsQ15_16s
//     ippsqAbsQ31_32s
*/

#include "ippsq.h"

#define resOK       0
#define resErr_f32  1
#define resErr_Q15  2
#define resErr_Q31  3

/******************************* Abs ******************************************/
static int test_Abs32f(void)
{
    Ipp32f  pSrc[8] = {0.0f, 1.0f, 2.0f, 32765.f, -32768.f, -5.f, -6.f, -7.f};
    Ipp32f  pDst[8];
    Ipp32f  pRef[8] = {0.0f, 1.0f, 2.0f, 32765.f,  32768.f,  5.f,  6.f,  7.f};
    int     i;
    ippsqAbs_32f(pSrc, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_f32;
    }
    return resOK;
}
static int test_Abs16s(void)
{
    Ipp16s  pSrc[8] = {0, 1, 2, 32765, -32768, -5, -6, -7};
    Ipp16s  pDst[8];
    Ipp16s  pRef[8] = {0, 1, 2, 32765,  32767,  5,  6,  7};
    int     i;
    ippsqAbsQ15_16s(pSrc, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q15;
    }
    return resOK;
}
static int test_Abs32s(void)
{
    Ipp32s  pSrc[8] = {0, 1, 2, 2147483645, (-2147483647-1), -5, -6, -7};
    Ipp32s  pDst[8];
    Ipp32s  pRef[8] = {0, 1, 2, 2147483645,  2147483647,      5,  6,  7};
    int     i;
    ippsqAbsQ31_32s(pSrc, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q31;
    }
    return resOK;
}

int main(void) {
    int result;
    result =  test_Abs32f();
    if (result != resOK) return result;
    result =  test_Abs16s();
    if (result != resOK) return result;
    result =  test_Abs32s();
    if (result != resOK) return result;
    return resOK;
}
/****************************************************************************/
