/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
// A simple example of performing AddC functions:
//     ippsqAddC_32f
//     ippsqAddCQ15_16s_s
//     ippsqAddCQ31_32s_s
*/

#include "ippsq.h"

#define resOK       0
#define resErr_f32  1
#define resErr_Q15  2
#define resErr_Q31  3

/******************************* AddC ******************************************/
static int test_AddC32f(void)
{
    Ipp32f  pSrc[8] = { 0.0f, 1.0f, 2.0f, 32765.f, -32765.f, -5.f,  -6.f, -7.f};
    Ipp32f  val     = 10.0f;
    Ipp32f  pDst[8];
    Ipp32f  pRef[8]  = {10.0f, 11.0f, 12.0f, 32775.f, -32755.f, 5.f, 4.f, 3.f};
    int     i;
    ippsqAddC_32f(pSrc, val, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_f32;
    }
    return resOK;
}
static int test_AddC16s(void)
{
    Ipp16s  pSrc[8] = { 0, 1, 2, 32765, -32765, -5,  -6, -7};
    Ipp16s  val     = 10;
    Ipp16s  pDst[8];
    Ipp16s  pRef[8] =  {10, 11, 12, 32767, -32755, 5, 4,  3};
    int     i;
    ippsqAddCQ15_16s_s(pSrc, val, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q15;
    }
    return resOK;
}
static int test_AddC32s(void)
{
    Ipp32s  pSrc[8] = { 0, 1, 2, 2147483645, -2147483645, -5,  -6, -7};
    Ipp32s  val     = 10;
    Ipp32s  pDst[8];
    Ipp32s  pRef[8]  = {10, 11, 12, 2147483647, -2147483635, 5, 4, 3};
    int     i;
    ippsqAddCQ31_32s_s(pSrc, val, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q31;
    }
    return resOK;
}

int main(void) {
    int result;
    result =  test_AddC32f();
    if (result != resOK) return result;
    result =  test_AddC16s();
    if (result != resOK) return result;
    result =  test_AddC32s();
    if (result != resOK) return result;
    return resOK;
}
/****************************************************************************/
