/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
// A simple example of performing Conj functions:
//     ippsqConj_32fc
//     ippsqConjQ15_16sc
//     ippsqConjQ31_32sc
*/

#include "ippsq.h"

#define resOK       0
#define resErr_f32  1
#define resErr_Q15  2
#define resErr_Q31  3

/******************************* Conj ******************************************/
static int test_Conj32f(void)
{
    Ipp32fc pSrc[4] = {{0.0f, 1.0f}, {2.0f, 32765.f}, {-32768.f, -5.f}, {-6.f, -7.f}};
    Ipp32fc pDst[4];
    Ipp32fc pRef[4] = {{0.0f, -1.0f}, {2.0f, -32765.f}, {-32768.f, 5.f}, {-6.f, 7.f}};
    int     i;
    ippsqConj_32fc(pSrc, pDst, 4);
    for (i = 0; i < 4; i++) {
        if ((pDst[i].re != pRef[i].re) || (pDst[i].im != pRef[i].im)) return resErr_f32;
    }
    return resOK;
}
static int test_Conj16s(void)
{
    Ipp16sc pSrc[4] = {{0, 1}, {2, 32765}, {-5, -32768}, {-6, -7}};
    Ipp16sc pDst[4];
    Ipp16sc pRef[4] = {{0, -1}, {2, -32765}, {-5, 32767}, {-6, 7}};
    int     i;
    ippsqConjQ15_16sc(pSrc, pDst, 4);
    for (i = 0; i < 4; i++) {
        if ((pDst[i].re != pRef[i].re) || (pDst[i].im != pRef[i].im)) return resErr_Q15;
    }
    return resOK;
}
static int test_Conj32s(void)
{
    Ipp32sc pSrc[4] = {{0, 1}, {2, 2147483645}, {-5, (-2147483647-1)}, {-6, -7}};
    Ipp32sc pDst[4];
    Ipp32sc pRef[4] = {{0, -1}, {2, -2147483645}, {-5, 2147483647}, {-6, 7}};
    int     i;
    ippsqConjQ31_32sc(pSrc, pDst, 4);
    for (i = 0; i < 4; i++) {
        if ((pDst[i].re != pRef[i].re) || (pDst[i].im != pRef[i].im)) return resErr_Q31;
    }
    return resOK;
}

int main(void) {
    int result;
    result =  test_Conj32f();
    if (result != resOK) return result;
    result =  test_Conj16s();
    if (result != resOK) return result;
    result =  test_Conj32s();
    if (result != resOK) return result;
    return resOK;
}
/****************************************************************************/
