/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
// A simple example of performing Mul functions:
//     ippsqMul_32f
//     ippsqMulQ15_16s_Rm
//     ippsqMulQ31_32s_Rm
*/

#include "ippsq.h"

#define resOK       0
#define resErr_f32  1
#define resErr_Q15  2
#define resErr_Q31  3

/******************************* Mul ******************************************/
static int test_Mul32f(void)
{
    Ipp32f  pSrc1[8] = { 0.0f, 1.0f, 2.0f, 32765.f, -32765.f, -5.f,  -6.f, -7.f};
    Ipp32f  pSrc2[8] = {-1.0f, 2.0f, 5.0f,    10.f,    -10.f,  3.f,  -4.f,  7.f};
    Ipp32f  pDst[8];
    Ipp32f  pRef[8]  = {0.0f, 2.0f, 10.0f, 327650.f, 327650.f, -15.f, 24.f, -49.f};
    int     i;
    ippsqMul_32f(pSrc1, pSrc2, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_f32;
    }
    return resOK;
}
static int test_Mul16s(void)
{
    Ipp16s  pSrc1[8] = { 0, 1*16384, 2*8192, 32765, -32765, -5*4096, -6*4096, -7*2048};
    Ipp16s  pSrc2[8] = {-1, 2*8192, 5*4096,     10,    -10,  3*8192, -4*4096,  7*2048};
    Ipp16s  pDst[8];
    Ipp16s  pRef[8] =  {0, 8192, 10240, 9, 9, -15360, 12288, -6272};
    int     i;
    ippsqMulQ15_16s_Rm(pSrc1, pSrc2, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q15;
    }
    return resOK;
}
static int test_Mul32s(void)
{
    Ipp32s  pSrc1[8] = { 0, 1*16384*16384, 2*8192*16384, 2147483645, -2147483645, -5*4096*16384, -6*4096*16384, -7*2048*16384};
    Ipp32s  pSrc2[8] = {-1, 2*8192*16384, 5*4096*16384,         10,          -10,  3*8192*16384, -4*4096*16384,  7*2048*16384};
    Ipp32s  pDst[8];
    Ipp32s  pRef[8]  = {0, 33554432, 41943040, 9, 9, -62914560, 50331648, -25690112};
    int     i;
    ippsqMulQ31_32s_Rm(pSrc1, pSrc2, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q31;
    }
    return resOK;
}

int main(void) {
    int result;
    result =  test_Mul32f();
    if (result != resOK) return result;
    result =  test_Mul16s();
    if (result != resOK) return result;
    result =  test_Mul32s();
    if (result != resOK) return result;
    return resOK;
}
/****************************************************************************/
