/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
// A simple example of performing MulC functions:
//     ippsqMul_32f
//     ippsqMulCQ15_16s_RmSfs
//     ippsqMulCQ31_32s_RmSfs
*/

#include "ippsq.h"

#define resOK       0
#define resErr_f32  1
#define resErr_Q15  2
#define resErr_Q31  3

/******************************* MulC ******************************************/
static int test_MulC32f(void)
{
    Ipp32f  pSrc[8] = { 0.0f, 1.0f, 2.0f, 32765.f, -32765.f, -5.f,  -6.f, -7.f};
    Ipp32f  val = 10.0f;
    Ipp32f  pDst[8];
    Ipp32f  pRef[8]  = {0.0f, 10.0f, 20.0f, 327650.f, -327650.f, -50.f, -60.f, -70.f};
    int     i;
    ippsqMul_32f(pSrc, val, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_f32;
    }
    return resOK;
}
static int test_MulC16s(void)
{
    Ipp16s  pSrc[8] = { 0, 1*16384, 2*8192, 32765, -32765, -5*4096, -6*4096, -7*2048};
    Ipp16s  val = 2*8192;
    int     scaleFactor = -1;
    Ipp16s  pDst[8];
    Ipp16s  pRef[8] =  {0, 16384, 16384, 32765, -32765, -20480, -24576, -14336};
    int     i;
    ippsqMulCQ15_16s_RmSfs(pSrc, val, pDst, 8, scaleFactor);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q15;
    }
    return resOK;
}
static int test_MulC32s(void)
{
    Ipp32s  pSrc[8] = { 0, 1*16384*16384, 2*8192*16384, 2147483645, -2147483645, -5*4096*16384, -6*4096*16384, -7*2048*16384};
    Ipp32s  val = 0x80000000;
    int     scaleFactor = 0;
    Ipp32s  pDst[8];
    Ipp32s  pRef[8]  = {0, -1*16384*16384, -2*8192*16384, -2147483645, 2147483645, 5*4096*16384, 6*4096*16384, 7*2048*16384};
    int     i;
    ippsqMulCQ31_32s_RmSfs(pSrc, val, pDst, 8, scaleFactor);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q31;
    }
    return resOK;
}

int main(void) {
    int result;
    result =  test_MulC32f();
    if (result != resOK) return result;
    result =  test_MulC16s();
    if (result != resOK) return result;
    result =  test_MulC32s();
    if (result != resOK) return result;
    return resOK;
}
/****************************************************************************/
