/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
// A simple example of performing PowerSpectr functions:
//     ippsqPowerSpectr_32fc
//     ippsqPowerSpectrQ15Q13_16sc_Rm
//     ippsqPowerSpectrQ31Q29_32sc_Rm
*/

#include "ippsq.h"

#define resOK       0
#define resErr_f32  1
#define resErr_Q15  2
#define resErr_Q31  3

/******************************* PowerSpectr ******************************************/
static int test_PowerSpectr32f(void)
{
    Ipp32fc pSrc[4] = {{0.0f, 1.0f}, {3.0f, -4.f}, {0.0, 0.0}, {-12.f, 5.f}};
    Ipp32f  pDst[4];
    Ipp32f  pRef[4] = {1.0f, 25.0f, 0.0f, 169.f};
    int     i;
    ippsqPowerSpectr_32fc(pSrc, pDst, 4);
    for (i = 0; i < 4; i++) {
        if (pDst[i] != pRef[i]) return resErr_f32;
    }
    return resOK;
}
static int test_PowerSpectr16s(void)
{
    Ipp16sc pSrc[4] = {{0, 16384}, {3 * 8192, -4 * 8192}, {-32768, -32768}, {-6, -7}};
    Ipp16s  pDst[4];
    Ipp16s  pRef[4] = {2048, 12800, 16384, 0};
    int     i;
    ippsqPowerSpectrQ15Q13_16sc_Rm(pSrc, pDst, 4);
    for (i = 0; i < 4; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q15;
    }
    return resOK;
}
static int test_PowerSpectr32s(void)
{
    Ipp32sc pSrc[4] = {{0, 1<<30}, {3 * (1<<29), -4 * (1<<29)}, {(-2147483647-1), (-2147483647-1)}, {-6, -7}};
    Ipp32s  pDst[4];
    Ipp32s  pRef[4] = {134217728, 838860800, 1073741824, 0};
    int     i;
    ippsqPowerSpectrQ31Q29_32sc_Rm(pSrc, pDst, 4);
    for (i = 0; i < 4; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q31;
    }
    return resOK;
}

int main(void) {
    int result;
    result =  test_PowerSpectr32f();
    if (result != resOK) return result;
    result =  test_PowerSpectr16s();
    if (result != resOK) return result;
    result =  test_PowerSpectr32s();
    if (result != resOK) return result;
    return resOK;
}
/****************************************************************************/
