/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
// A simple example of performing Rms functions:
//     ippsqRms_32f
//     ippsqRmsQ15_16s_Rz
//     ippsqRmsQ31_32s_Rz
*/

#include "ippsq.h"

#define resOK       0
#define resErr_f32  1
#define resErr_Q15  2
#define resErr_Q31  3

/******************************* Rms ******************************************/
static int test_Rms32f(void)
{
    Ipp32f  pSrc[4] = { 0.0f, -3.0f, 4.0f, -12.f};
    Ipp32f  pDst;
    Ipp32f  pRef  = 6.5f;
    ippsqRms_32f(pSrc, 4, &pDst);
    if (pDst != pRef) return resErr_f32;
    return resOK;
}
static int test_Rms16s(void)
{
    Ipp16s  pSrc[4] = { 0, -3*2048, 4*2048, -12*2048};
    Ipp16s  pDst;
    Ipp16s  pRef = 13312;
    ippsqRmsQ15_16s_Rz(pSrc, 4, &pDst);
    if (pDst != pRef) return resErr_Q15;
    return resOK;
}
static int test_Rms32s(void)
{
    Ipp32s  pSrc[4] = { 0, -3*(1<<27), 4*(1<<27), -12*(1<<27)};
    Ipp32s  pDst;
    Ipp32s  pRef = 872415232;
    ippsqRmsQ31_32s_Rz(pSrc, 4, &pDst);
    if (pDst != pRef) return resErr_Q31;
    return resOK;
}

int main(void) {
    int result;
    result =  test_Rms32f();
    if (result != resOK) return result;
    result =  test_Rms16s();
    if (result != resOK) return result;
    result =  test_Rms32s();
    if (result != resOK) return result;
    return resOK;
}
/****************************************************************************/
