/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
// A simple example of performing Set functions:
//     ippsqSet_32f
//     ippsqSetQ15_16s
//     ippsqSetQ31_32s
*/

#include "ippsq.h"

#define resOK       0
#define resErr_f32  1
#define resErr_Q15  2
#define resErr_Q31  3

/******************************* Set ******************************************/
static int test_Set32f(void)
{
    Ipp32f  val     = 10.0f;
    Ipp32f  pDst[8];
    Ipp32f  pRef[8]  = {10.0f, 10.0f, 10.0f, 10.0f, 10.0f, 10.0f, 10.0f, 10.0f};
    int     i;
    ippsqSet_32f(val, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_f32;
    }
    return resOK;
}
static int test_Set16s(void)
{
    Ipp16s  val     = 10;
    Ipp16s  pDst[8];
    Ipp16s  pRef[8] =  {10, 10, 10, 10, 10, 10, 10, 10};
    int     i;
    ippsqSetQ15_16s(val, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q15;
    }
    return resOK;
}
static int test_Set32s(void)
{
    Ipp32s  val     = 10;
    Ipp32s  pDst[8];
    Ipp32s  pRef[8]  = {10, 10, 10, 10, 10, 10, 10, 10};
    int     i;
    ippsqSetQ31_32s(val, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q31;
    }
    return resOK;
}

int main(void) {
    int result;
    result =  test_Set32f();
    if (result != resOK) return result;
    result =  test_Set16s();
    if (result != resOK) return result;
    result =  test_Set32s();
    if (result != resOK) return result;
    return resOK;
}
/****************************************************************************/
