/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
// A simple example of performing RShiftC functions:
//     ippsqRShiftCQ15_16s_Rm
//     ippsqRShiftCQ31_32s_Rm
*/

#include "ippsq.h"

#define resOK       0
#define resErr_Q15  2
#define resErr_Q31  3

/******************************* RShiftC ******************************************/
static int test_RShiftC16s(void)
{
    Ipp16s  pSrc[8] = { 0, 1, 2, 32765, -32765, -5, -6, -7};
    Ipp16s  val     = 1;
    Ipp16s  pDst[8];
    Ipp16s  pRef[8] =  {0, 0, 1, 16382, -16383, -3, -3, -4};
    int     i;
    ippsqRShiftCQ15_16s_Rm(pSrc, val, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q15;
    }
    return resOK;
}
static int test_RShiftC32s(void)
{
    Ipp32s  pSrc[8] = { 0, 1, 2, 2147483645, -2147483645, -5,  -6, -7};
    Ipp32s  val     = 1;
    Ipp32s  pDst[8];
    Ipp32s  pRef[8]  = {0, 0, 1, 1073741822, -1073741823, -3, -3, -4};
    int     i;
    ippsqRShiftCQ31_32s_Rm(pSrc, val, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q31;
    }
    return resOK;
}

int main(void) {
    int result;
    result =  test_RShiftC16s();
    if (result != resOK) return result;
    result =  test_RShiftC32s();
    if (result != resOK) return result;
    return resOK;
}
/****************************************************************************/
