/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
// A simple example of performing Sub functions:
//     ippsqSub_32f
//     ippsqSubQ15_16s_s
//     ippsqSubQ31_32s_s
*/

#include "ippsq.h"

#define resOK       0
#define resErr_f32  1
#define resErr_Q15  2
#define resErr_Q31  3

/******************************* Sub ******************************************/
static int test_Sub32f(void)
{
    Ipp32f  pSrc1[8] = { 0.0f, 1.0f, 2.0f, 32765.f, -32765.f, -5.f,  -6.f, -7.f};
    Ipp32f  pSrc2[8] = {-1.0f, 2.0f, 5.0f,    10.f,    -10.f,  3.f,  -4.f,  7.f};
    Ipp32f  pDst[8];
    Ipp32f  pRef[8]  = {-1.0f, 1.0f, 3.0f, -32755.f, 32755.f, 8.f, 2.f, 14.f};
    int     i;
    ippsqSub_32f(pSrc1, pSrc2, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_f32;
    }
    return resOK;
}
static int test_Sub16s(void)
{
    Ipp16s  pSrc1[8] = { 0, 1, 2, 32765, -32765, -5,  -6, -7};
    Ipp16s  pSrc2[8] = {-1, 2, 5,   -10,     10,  3,  -4,  7};
    Ipp16s  pDst[8];
    Ipp16s  pRef[8] =  {-1, 1, 3, -32768, 32767, 8, 2, 14};
    int     i;
    ippsqSubQ15_16s_s(pSrc1, pSrc2, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q15;
    }
    return resOK;
}
static int test_Sub32s(void)
{
    Ipp32s  pSrc1[8] = { 0, 1, 2, 2147483645, -2147483645, -5,  -6, -7};
    Ipp32s  pSrc2[8] = {-1, 2, 5,        -10,          10,  3,  -4,  7};
    Ipp32s  pDst[8];
    Ipp32s  pRef[8]  = {-1, 1, 3, (-2147483647-1), 2147483647, 8, 2, 14};
    int     i;
    ippsqSubQ31_32s_s(pSrc1, pSrc2, pDst, 8);
    for (i = 0; i < 8; i++) {
        if (pDst[i] != pRef[i]) return resErr_Q31;
    }
    return resOK;
}

int main(void) {
    int result;
    result =  test_Sub32f();
    if (result != resOK) return result;
    result =  test_Sub16s();
    if (result != resOK) return result;
    result =  test_Sub32s();
    if (result != resOK) return result;
    return resOK;
}
/****************************************************************************/
