/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
// A simple example of performing SumSqr functions:
//     ippsqSumSqr_32f
//     ippsqSumSqrQ15Q30_16s64s
//     ippsqSumSqrQ31Q48_32s64s_Rms
*/

#include "ippsq.h"

#define resOK       0
#define resErr_f32  1
#define resErr_Q15  2
#define resErr_Q31  3

/******************************* SumSqr ******************************************/
static int test_SumSqr32f(void)
{
    Ipp32f  pSrc[8] = {0.0f, 2.0f, 4.0f, 32.f, -32.f, -5.f,  -6.f, -7.f};
    Ipp32f  pDst;
    Ipp32f  pRef  = 2178.f;
    ippsqSumSqr_32f(pSrc, 8, &pDst);
    if (pDst != pRef) return resErr_f32;
    return resOK;
}
static int test_SumSqr16s(void)
{
    Ipp16s  pSrc[8] = {0, 2, 4, 32765, -32765, -5,  -6, -7};
    Ipp64s  pDst;
    Ipp64s  pRef = 2147090580;
    ippsqSumSqrQ15Q30_16s64s(pSrc, 8, &pDst);
    if (pDst != pRef) return resErr_Q15;
    return resOK;
}
static int test_SumSqr32s(void)
{
    Ipp32s  pSrc[8] = {0, 2, 4, 32765, -32765, -5,  -6, -7};
    Ipp64s  pDst;
    Ipp64s  pRef = 131048;
    ippsqSumSqrQ31Q48_32s64s_Rms(pSrc, 8, &pDst);
    if (pDst != pRef) return resErr_Q31;
    return resOK;
}

int main(void) {
    int result;
    result =  test_SumSqr32f();
    if (result != resOK) return result;
    result =  test_SumSqr16s();
    if (result != resOK) return result;
    result =  test_SumSqr32s();
    if (result != resOK) return result;
    return resOK;
}
/****************************************************************************/
