/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
// A simple example of performing Variance functions:
//     ippsqVariance_32f
//     ippsqVarianceQ15_16s32s_Rz
//     ippsqVarianceQ31_32s64s_Rz
*/

#include "ippsq.h"

#define resOK       0
#define resErr_f32  1
#define resErr_Q15  2
#define resErr_Q31  3

/******************************* Variance ******************************************/
static int test_Variance32f(void)
{
    Ipp32f  pSrc[5] = {1.0f, 2.0f, 3.0f, 3.0f, 1.f};
    Ipp32f  pDst;
    Ipp32f  pRef  = 1.0f;
    ippsqVariance_32f(pSrc, 5, &pDst);
    if (pDst != pRef) return resErr_f32;
    return resOK;
}
static int test_Variance16s(void)
{
    Ipp16s  pSrc[5] = {1*8192, 2*8192, 3*8192, 3*8192, 1*8192};
    Ipp32s  pDst;
    Ipp32s  pRef = (8192 * 8192) >> 15;
    ippsqVarianceQ15_16s32s_Rz(pSrc, 5, &pDst);
    if (pDst != pRef) return resErr_Q15;
    return resOK;
}
static int test_Variance32s(void)
{
    Ipp32s  pSrc[5] = {1*(1<<29), 2*(1<<29), 3*(1<<29), 3*(1<<29), 1*(1<<29)};
    Ipp64s  pDst;
    Ipp64s  pRef = ((Ipp64s)(1<<29) * (Ipp64s)(1<<29)) >> 31;
    ippsqVarianceQ31_32s64s_Rz(pSrc, 5, &pDst);
    if (pDst != pRef) return resErr_Q31;
    return resOK;
}

int main(void) {
    int result;
    result =  test_Variance32f();
    if (result != resOK) return result;
    result =  test_Variance16s();
    if (result != resOK) return result;
    result =  test_Variance32s();
    if (result != resOK) return result;
    return resOK;
}
/****************************************************************************/
